package vishnu.server;

/*********************************************************************
 Copyright (C) 2003 Imperial College London

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
***********************************************************************/

//import org.apache.xpath.XPathAPI;
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 
import org.xml.sax.InputSource;
import javax.xml.parsers.*;

import java.io.*;
import java.util.*;
import vishnu.datablock.*;
import java.awt.*;

public class Packaging
{
    static Document doc = null;
    static boolean out = false;

    static String matrixString = "";

    public Packaging()
    {
    }


    public static int[] inInts(StringTokenizer st)
    {
	String line;
	int intCount;

	if (!st.hasMoreTokens()) {
	    return new int[0];
	}

	intCount = Integer.parseInt(st.nextToken());
	if(out) System.out.println(intCount);

	int[] intarray = new int[intCount];
	for (int c = 0; c < intCount;c++)
	    {
		line = st.nextToken();
		int n = Integer.parseInt(line);
		intarray[c] = n;
		if(out) System.out.println(n);
	    }
	return intarray;
    }		
    
    
    public static String [] inStrings(StringTokenizer st)
    {
	String line;
	String [] instrings;
	int strCount;

	if (!st.hasMoreTokens()) {
	    return new String[0];
	}
	
	strCount = Integer.parseInt(st.nextToken());
	if(out)	System.out.println(strCount);
	   
	instrings = new String[strCount];
	for (int c = 0; c < strCount;c++)
	    {
		instrings[c] = st.nextToken();
	    }

	return instrings;
    }

    public static SparseMatrix inMatrix(StringTokenizer st, int wordCount)
    {
	String line;
	int rowCount,column,value;

	if (!st.hasMoreTokens()) {
	    return new SparseMatrix(0, wordCount);
	}

	rowCount = Integer.parseInt(st.nextToken());
	matrixString += String.valueOf(rowCount) + "\n";
	if(out) System.out.println(rowCount);
	
	SparseMatrix matrix = new SparseMatrix(rowCount,wordCount);
	
	for (int c = 0; c < rowCount;c++)
	    {
		line = st.nextToken();
		matrixString += line + "\n";

		if(out) System.out.println(line);
		
		if(line.length() > 1)
		    {
			StringTokenizer tokens = new StringTokenizer(line,",");
			
			while(tokens.hasMoreTokens())
			    {
				String tokenString = tokens.nextToken();
				value = Integer.parseInt(tokenString);
				
				if(tokens.hasMoreTokens())
				    { 
					tokenString = tokens.nextToken();
					column = Integer.parseInt(tokenString);
					matrix.addDataItem(c, column, value);
				    }
			    }// while				
		    } //if
	    } // for 
	return matrix;
    } // inMatrix

    
    public static Vector [] inCluster(StringTokenizer st, int docCount)
    {
	String line;
	int clusterCount,value;
	
	if (!st.hasMoreTokens()) {
	    return new Vector[0];
	}
	clusterCount = Integer.parseInt(st.nextToken());
	if(out)	System.out.println(clusterCount);
	Vector [] clusters = new Vector[clusterCount];
	
	for (int c = 0; c < clusterCount;c++)
	    {
		Vector v = new Vector();

		line = st.nextToken();
		if(out) System.out.println(line);
		if (line.length()>0)
		    {
			StringTokenizer tokens = new StringTokenizer(line,",");
			
			while(tokens.hasMoreTokens())
			    {
				String tokenString = tokens.nextToken();
				
				value = Integer.parseInt(tokenString);
				if(value >= 0 && value <= docCount)
				    {
					v.addElement(new Integer(value));
				    } // if
			    } // while
		    } // if
		
		clusters[c]=v;
	    } // for
	return clusters;
    }
 

    public static Point2D[] inPoint2D(StringTokenizer st)
    {
	String line;
	int count ;
	float x = 0,y = 0;
	
	if (!st.hasMoreTokens()) {
	    return new Point2D[0];
	}
	
	count = Integer.parseInt(st.nextToken());
	if(out) System.out.println(count);
	
	Point2D [] array = new Point2D[count];
	
	for (int c = 0; c < count;c++)
	    {
		line = st.nextToken();
		if(out)	System.out.println(line);
		StringTokenizer tokens = new StringTokenizer(line,",");

		if(tokens.hasMoreTokens())
		    {
			String tokenString = tokens.nextToken();
			x = Float.parseFloat(tokenString);
		    }
			
		if(tokens.hasMoreTokens())
		    {
			String tokenString = tokens.nextToken();
			y = Float.parseFloat(tokenString);
		    } // if
		    		
		array[c]= new Point2D(x,y);
	    } // for
	return array;
    }

 
 public Element getXMLPackage(String result, Vector desc) 
    {
	String [] words;
	int [] docs;
	String [] descriptions;
	SparseMatrix matrix;
	Vector [] clusters;
	Point2D [] sammon;

	StringTokenizer st = new StringTokenizer(result);

	docs = inInts(st);
	words = inStrings(st);
	matrix = inMatrix(st,words.length);
	clusters = inCluster(st, docs.length);
	sammon = inPoint2D(st);	
	descriptions = new String[docs.length];

	Element response = doc.createElement("response");
	response.setAttribute("from","Packaging");
	
	Element docList = doc.createElement("docNodeList");
	response.appendChild(docList);
	for( int i = 0; i < docs.length; i++ )
	    {
		Element node = doc.createElement("docNode");
		node.setAttribute("docNum", String.valueOf(docs[i]));
		node.setAttribute("title",(String)desc.elementAt(i));
		docList.appendChild(node);
	    }

	Element sammonList = doc.createElement("sammonList");
	response.appendChild(sammonList);
	for( int i = 0; i < sammon.length; i++ )
	    {
		float x = sammon[i].x;
		float y = sammon[i].y;		
		Element node = doc.createElement("sammonNode");
		node.setAttribute("xpos",String.valueOf(x));
		node.setAttribute("ypos",String.valueOf(y));
		sammonList.appendChild(node);
	    }

	Element wordList = doc.createElement("wordList");
	response.appendChild(wordList);
	for( int i = 0; i < words.length; i++ )
	    {
		Element node = doc.createElement("wordNode");
		node.setAttribute("term",words[i]);
		wordList.appendChild(node);
	    }

	Element mat = doc.createElement("DocTermSparseMatrix");
	response.appendChild(mat);

	st = new StringTokenizer(matrixString);
	String line;
	int rowCount,column,value;

	rowCount = Integer.parseInt(st.nextToken());
	mat.setAttribute("rows",String.valueOf(rowCount));
	mat.setAttribute("cols",String.valueOf(words.length));

	Element rowList = doc.createElement("rowList");
	mat.appendChild(rowList);

	for (int c = 0; c < rowCount;c++)
	    {
		line = st.nextToken();
		if(line.length() > 1)
		    {
			Element colList = doc.createElement("colList");
			rowList.appendChild(colList);
			colList.setAttribute("row",String.valueOf(c));
			
			StringTokenizer tokens = new StringTokenizer(line,",");
			
			while(tokens.hasMoreTokens())
			    {
				Element entryElem = doc.createElement("entry");
				colList.appendChild(entryElem);

				String tokenString = tokens.nextToken();
				value = Integer.parseInt(tokenString);

				entryElem.setAttribute("docFreq",tokenString);
				
				if(tokens.hasMoreTokens())
				    { 
					tokenString = tokens.nextToken();
					column = Integer.parseInt(tokenString);
					entryElem.setAttribute("term",tokenString);
				    }
			    }// while				
		    } //if
	    } // for 

	Element clusterList = doc.createElement("ClusterList");

	response.appendChild(clusterList);

	for( int i = 0; i < clusters.length; i++ )
	    {
		Element cluster = doc.createElement("Cluster");
		clusterList.appendChild(cluster);

		Vector vector = clusters[i];
		for( int c = 0; c < vector.size(); c++ )
		    {
			Element entry = doc.createElement("leaf");
			cluster.appendChild(entry);
			
			Integer I = (Integer)vector.elementAt(c);
			entry.setAttribute("docNum",String.valueOf(I.intValue()));
		    }
	    }

	for( int i = 0; i < docs.length; i++ ){
            descriptions[i] = (String)desc.elementAt(i);
	}

	return response;
    }


   
    public DataBlock getDataPackage(String result, Vector desc) 
    {
	String [] words;
	int [] docs;
	String [] descriptions;
	SparseMatrix matrix;
	Vector [] clusters;
	Point2D [] sammon;

	StringTokenizer st = new StringTokenizer(result);

	docs = inInts(st);
	words = inStrings(st);
	matrix = inMatrix(st,words.length);
	clusters = inCluster(st, docs.length);
	sammon = inPoint2D(st);	
	descriptions = new String[docs.length];

	for( int i = 0; i < docs.length; i++ )
	    {
		descriptions[i] = (String)desc.elementAt(i);
		//descriptions[i]	= "doc"+Integer.toString(i);
	    }

	DataBlock data = new DataBlock(docs,descriptions,words,matrix,clusters,sammon); 
	return data;
    }

}







